#include <wx/wx.h>
#include <wx/txtstrm.h>
#include "calc.h"

#define CHECK_INTERVAL	20
#define TIMEOUT		500

calc::calc(const wxString &cmd) {
	cp = wxProcess::Open(_("bc -ql"));
	if (!cp) throw calc_error(_("Ne mogu pokrenuti bc!"));

	exec(_("pi = a(1)*4"));
	registerfunction(_("mod"), _("a,b"), _("auto t,r; t=scale; scale=0; r=a%b; scale=t; return r;"));
	exec(cmd);
}

calc::~calc() {
	exec(_("quit"));
}

void calc::exec(const wxString &cmd) {
	if (cmd.empty()) return;
	
	wxOutputStream *os = cp->GetOutputStream();
	if (!os) throw calc_error(_("exec: Ne mogu dobiti output stream!"));
	wxTextOutputStream tos(*os);
	tos << cmd << endl;
}

double calc::getdbl() {
	for (int n = 0; !cp->IsInputAvailable() && !cp->IsErrorAvailable() && n < TIMEOUT / CHECK_INTERVAL; ++n) wxMilliSleep(CHECK_INTERVAL);
	if (cp->IsErrorAvailable()) {
		wxInputStream *eis = cp->GetErrorStream();
		if (!eis) throw calc_error(_("getdbl: Ne mogu dobiti error stream!"));
		wxTextInputStream etis(*eis);
		wxString err;
		while (cp->IsErrorAvailable())
			err += etis.ReadLine() + _("\n");
		if (err.size()) err.erase(err.size()-1);
		throw calc_error(_("bc:\n") + err);
	}
	
	if (!cp->IsInputAvailable()) throw calc_error(_("Nisam docekao podatke!"));
	
	wxInputStream *is = cp->GetInputStream();
	if (!is) throw calc_error(_("getdbl: Ne mogu dobiti input stream!"));

	double d;
	wxTextInputStream tis(*is);
	wxString lin = tis.ReadLine();
	if (!lin.ToDouble(&d)) throw calc_error(_("Not a number: ") + lin);
	return d;
}

bool calc::hasdata() const {
	return cp->IsInputAvailable();
}

void calc::registerfunction(const wxString &name, const wxString &arguments, const wxString &body, bool isvoid) {
	wxString cmd = _("define ");
	if (isvoid) cmd += _("void ");
	cmd += name + _("(") + arguments + _(") {\n") + body + _("}");
	exec(cmd);
}
