#include <wx/wx.h>
#include <wx/xml/xml.h>
#include <wx/aboutdlg.h>
#include <wx/generic/aboutdlgg.h>
#include "forijer.h"
#include "gnuplot.h"
#include "calc.h"

#include "forijer.xpm"

using namespace std;

BEGIN_EVENT_TABLE( ForijerFrame, wxFrame )
	EVT_MENU(Menu_File_Quit, ForijerFrame::OnQuit)
	EVT_MENU(Menu_File_New, ForijerFrame::OnNew)
	EVT_MENU(Menu_File_Open, ForijerFrame::OnOpen)
	EVT_MENU(Menu_File_Save, ForijerFrame::OnSave)
	EVT_MENU(Menu_File_Save_As, ForijerFrame::OnSaveAs)
	EVT_MENU(Menu_Options_GnuplotParametri, ForijerFrame::OnGnuplotParametri)
	EVT_MENU(Menu_Options_GnuplotConsole, ForijerFrame::OnShowGPConsole)
	EVT_MENU(Menu_Help_About, ForijerFrame::OnAbout)
	EVT_MENU(Menu_Help_About_bc, ForijerFrame::OnAboutBC)
	EVT_MENU(Menu_Help_About_gnuplot, ForijerFrame::OnAboutGnuplot)
	EVT_BUTTON(id_Nacrtaj, ForijerFrame::OnNacrtaj)
	EVT_SPINCTRL(id_Spin, ForijerFrame::OnSpin)
END_EVENT_TABLE();

const wxString forijer_wildcards = _("Forijer datoteke (*.frj)|*.frj|Sve datoteke|*.*");

IMPLEMENT_APP(Forijerapp);

wxString Forijerapp::CollectExecOutput(const wxString &cmd) {
	wxArrayString dat;
	wxString r;
	if (wxExecute(cmd, dat)) return wxString();
	for (unsigned int i = 0; i < dat.size(); ++i)
		r+=dat[i]+_("\n");
	return r;
}

wxString Forijerapp::GetBCVersion() {
	return CollectExecOutput(_("bc -v"));
}

wxString Forijerapp::GetGPVersion() {
	return CollectExecOutput(_(GNUPLOTCMD " --version"));
}

bool Forijerapp::OnInit() {
	wxString bcv = GetBCVersion();
	if (bcv.empty()) {
		fprintf(stderr, "bc nije nadjen!\n");
		return false;
	}

	printf("wxWidgets version: %d.%d.%d\n\n", wxMAJOR_VERSION, wxMINOR_VERSION, wxRELEASE_NUMBER);

	printf("bc version: %s\n", (const char *)bcv.mb_str(wxConvUTF8));

	wxString gpv = GetGPVersion();
	if (gpv.empty()) {
		fprintf(stderr, "gnuplot nije nadjen!\n");
		return false;
	}
	
	printf("gnuplot version: %s\n", (const char *)gpv.mb_str(wxConvUTF8));
	

	gp = OpenGnuplot();
	if (!gp) {
		fprintf(stderr, "Ne mogu inicijalizirati gnuplot!\n");
		return false;
	}

	printf("Pokrecem Forijer-a...\n");
	
	ForijerFrame *frame = new ForijerFrame(gp);
	frame->Show();
	SetTopWindow(frame);
	
	return true;
}

int Forijerapp::OnExit() {
	if (gp) delete gp;

	printf("Pa, pa!\n");
	
	return wxApp::OnExit();
}




ForijerFrame::ForijerFrame(gnuplot *_gp)
	: wxFrame((wxFrame *)NULL, -1, _("Forijer")),
	  dlgFileOpen(this, _("Otvori datoteku..."), wxEmptyString, wxEmptyString, forijer_wildcards, wxFD_OPEN | wxFD_FILE_MUST_EXIST),
	  dlgFileSave(this, _("Spremi datoteku kao..."), wxEmptyString, wxEmptyString, forijer_wildcards, wxFD_SAVE | wxFD_OVERWRITE_PROMPT),
	  gp(_gp)
{
	///// Init frame-a
	SetIcon(forijer_xpm);
	
	wxMenu *menuFile = new wxMenu;
	menuFile->Append(Menu_File_New, _("&Novo\tCtrl+N"));
	menuFile->Append(Menu_File_Open, _("&Otvori...\tCtrl+O"));
	menuFile->Append(Menu_File_Save, _("&Spremi\tCtrl+S"));
	menuFile->Append(Menu_File_Save_As, _("Spremi k&ao..."));
	menuFile->AppendSeparator();
	menuFile->Append(Menu_File_Quit, _("E&xit\tCtrl+Q"));

	wxMenu *menuOptions = new wxMenu;
	chkSimplify = menuOptions->AppendCheckItem(Menu_Options_PojednostavniFormulu, _("&Pojednostavni formulu"));
	chkShowF = menuOptions->AppendCheckItem(Menu_Options_PrikazuFormulu, _("Prikazi &formulu"));
	chkShowFiC = menuOptions->AppendCheckItem(Menu_Options_PrikaziFormuluUKonzoli, _("Prikazi f&ormulu u konzoli"));
	menuOptions->Append(Menu_Options_GnuplotParametri, _("&Gnuplot parametri\tCtrl+G"));
	menuOptions->Append(Menu_Options_GnuplotConsole, _("Gnuplot &konzola\tCtrl+K"));
	
	wxMenu *menuHelp = new wxMenu;
	menuHelp->Append(Menu_Help_About, _("O &programu...\tCtrl+P"));
	menuHelp->Append(Menu_Help_About_bc, _("O &bc..."));
	menuHelp->Append(Menu_Help_About_gnuplot, _("O &gnuplot..."));
	
	wxMenuBar *menuBar = new wxMenuBar;
	menuBar->Append(menuFile, _("&File"));
	menuBar->Append(menuOptions, _("&Options"));
	menuBar->Append(menuHelp, _("&Help"));
	
	SetMenuBar(menuBar);

	wxBoxSizer *mainSizer = new wxBoxSizer(wxVERTICAL);

	wxFlexGridSizer *dSizer = new wxFlexGridSizer(2);

	dSizer->Add(new wxStaticText(this, -1, _("a0:")), 0, wxALIGN_CENTER_VERTICAL);
	dSizer->Add(txtA0 = new wxTextCtrl(this, -1, wxString(), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE), 1, wxEXPAND | wxALL, 1);
	txtA0->MarkDirty();
	dSizer->AddGrowableRow(0, 1);

	dSizer->Add(new wxStaticText(this, -1, _("a(n):")), 0, wxALIGN_CENTER_VERTICAL);
	dSizer->Add(txtAn = new wxTextCtrl(this, -1, wxString(), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE), 1, wxEXPAND | wxALL, 1);
	txtAn->MarkDirty();
	dSizer->AddGrowableRow(1, 1);

	dSizer->Add(new wxStaticText(this, -1, _("b(n):")), 0, wxALIGN_CENTER_VERTICAL);
	dSizer->Add(txtBn = new wxTextCtrl(this, -1, wxString(), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE), 1, wxEXPAND | wxALL, 1);
	txtBn->MarkDirty();
	dSizer->AddGrowableRow(2, 1);

	dSizer->Add(new wxStaticText(this, -1, _("Preciznost:")), 0, wxALIGN_CENTER_VERTICAL);
	wxBoxSizer *bs = new wxBoxSizer(wxHORIZONTAL);
	{
		chkRedrawAtChange = NULL;
		bs->Add(spnPrec = new wxSpinCtrl(this, id_Spin, wxString(), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 100, 1));
		bs->Add(chkRedrawAtChange = new wxCheckBox(this, -1, _("Crtaj pri promjeni")), 1, wxEXPAND);
		chkRedrawAtChange->SetValue(true);
	}
	dSizer->Add(bs, 0, wxEXPAND);
	dSizer->AddGrowableCol(1);

	mainSizer->Add(dSizer, 1, wxEXPAND);
	mainSizer->Add(0, 10);
	mainSizer->Add(new wxButton(this, id_Nacrtaj, _("Nacrtaj!")), 0, wxALIGN_CENTER_HORIZONTAL);
		
	SetSizerAndFit(mainSizer);
	

	///// Init plot podataka
	plotdata.a0 = 0.;
	plotdata.an.push_back(0.);
	plotdata.bn.push_back(0.);

	gnuplot_params = _("s(x)");

	///// Init konzole
	gpc = new GPConsole(this, gp);
}

void ForijerFrame::ErrorBox(const wxString &msg, const wxString &title) {
	wxMessageBox(msg, title, wxICON_ERROR | wxOK, this);
}

bool ForijerFrame::ParseFnArray(vector<double> &list, const wxString &func, unsigned int size) {
	list.clear();

	try {
		wxString cmd;
		calc c;
		
#ifndef __WINDOWS__
		c.registerfunction(_("f"), _("n"), func, true);
#else
		c.registerfunction(_("f"), _("n"), func);
#endif

		cmd.Printf(_("for (i=1;i<=%u;++i) f(i)"), size);
		c.exec(cmd);
		
		for (unsigned int i = 0; i < size; ++i) {
			double d = c.getdbl();
			list.push_back(d);
		}
	} catch (calc_error e) {
		ErrorBox(e.GetMsg(), &list == &plotdata.an ? _("an error:") : _("bn error:"));
		return false;
	}

	return true;
}

bool ForijerFrame::ProzvaciPodatke() {
	unsigned int size = spnPrec->GetValue();
	
	if (txtA0->IsModified()) {
		try {
			calc c(txtA0->GetValue());
			plotdata.a0 = c.getdbl();
		} catch (calc_error e) {
			ErrorBox(e.GetMsg(), _("a0 error:"));
			return false;
		}
		txtA0->DiscardEdits();
	}

	if (txtAn->IsModified() || plotdata.an.size() != size) {
		if (!ParseFnArray(plotdata.an, txtAn->GetValue(), size))
			return false;
		txtAn->DiscardEdits();
	}

	if (txtBn->IsModified() || plotdata.bn.size() != size) {
		if (!ParseFnArray(plotdata.bn, txtBn->GetValue(), size))
			return false;
		txtBn->DiscardEdits();
	}

	return true;
}

void ForijerFrame::Nacrtaj() {
	if (!ProzvaciPodatke()) return;

	bool simplify = chkSimplify->IsChecked();
	wxString fxcmd;
	fxcmd.Printf(_("s(x)=%g"), plotdata.a0 / 2);
	
	for (int i = 0; i < spnPrec->GetValue(); ++i) {
		wxString det;
		if (!simplify || (plotdata.an[i] != 0.)) {
			det.Printf(_("+%g*cos(%d*x)"), plotdata.an[i], i+1);
			fxcmd+=det;
		}
		if (!simplify || (plotdata.bn[i] != 0.)) {
			det.Printf(_("+%g*sin(%d*x)"), plotdata.bn[i], i+1);
			fxcmd+=det;
		}
	}

	if (chkShowF->IsChecked()) {
		wxMessageBox(fxcmd, _("Formula:"), wxOK, this);
	}
	if (chkShowFiC->IsChecked()) {
		gpc->ExecutedCmd(fxcmd);
	}
	
	gp->sendcmd(fxcmd);

	if (gnuplot_params.size()) {
		const wxString pcmd = _("plot ") + gnuplot_params;
		gp->sendcmd(pcmd);
		gpc->ExecutedCmd(pcmd);
	}
}

bool ForijerFrame::LoadFormulas(const wxString &fname) {
	wxXmlDocument doc;
	if (!doc.Load(fname)) {
		ErrorBox(_("Ne mogu ucitati datoteku!"), _("XML error"));
		return false;
	}
	
	if (doc.GetRoot()->GetName() != _("forijer")) {
		ErrorBox(_("Datoteka je krivog formata!"), _("XML error"));
		return false;
	}
	
	int ima = 0;
	wxXmlNode *child = doc.GetRoot()->GetChildren();
	for (;child;child=child->GetNext()) {
		if (child->GetType() == wxXML_COMMENT_NODE) continue;
		
		if (child->GetName() == _("a0")) {
			ima |= 1;
			txtA0->SetValue(child->GetNodeContent());
			txtA0->MarkDirty();
		} else if (child->GetName() == _("an")) {
			ima |= 2;
			txtAn->SetValue(child->GetNodeContent());
			txtAn->MarkDirty();
		} else if (child->GetName() == _("bn")) {
			ima |= 4;
			txtBn->SetValue(child->GetNodeContent());
			txtBn->MarkDirty();
		} else {
			fprintf(stderr, "Nepoznat nod u datoteci: %s\n", (const char *)child->GetName().mb_str(wxConvUTF8));
		}
	}
	
	if (ima != 7) {
		fprintf(stderr, "Nedostaju formule u datoteci!\n");
	}
	
	return true;
}

bool ForijerFrame::SaveFormulas(const wxString &fname) {
	wxXmlDocument doc;
	wxXmlNode *rut = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, _("forijer"));
	
	doc.SetRoot(rut);
	
	wxXmlNode *nod;
	
	nod = new wxXmlNode(rut, wxXML_ELEMENT_NODE, _("a0"));
	new wxXmlNode(nod, wxXML_CDATA_SECTION_NODE, wxEmptyString, txtA0->GetValue());
	
	nod = new wxXmlNode(rut, wxXML_ELEMENT_NODE, _("an"));
	new wxXmlNode(nod, wxXML_CDATA_SECTION_NODE, wxEmptyString, txtAn->GetValue());
	
	nod = new wxXmlNode(rut, wxXML_ELEMENT_NODE, _("bn"));
	new wxXmlNode(nod, wxXML_CDATA_SECTION_NODE, wxEmptyString, txtBn->GetValue());
	
	if (!doc.Save(fname)) {
		ErrorBox(_("Ne mogu spremiti datoteku!"), _("XML save error"));
		return false;
	}
	return true;
}

void ForijerFrame::OnQuit( wxCommandEvent& WXUNUSED( event ) ) {
	Close();
}

void ForijerFrame::OnAbout( wxCommandEvent& WXUNUSED( event ) ) {
	wxAboutDialogInfo about;
	about.SetVersion(_("0.0.2"));
	about.SetDescription(_("Programchich za iscrtavanje Fourierovih funkcija prema koeficijentima."));
	about.AddDeveloper(_("Davor Cihlar"));
	about.SetWebSite(_("http://davor.cihlar.biz/projects/fourier/"));
	wxGenericAboutBox(about);
}

void ForijerFrame::OnAboutBC(wxCommandEvent &) {
	wxMessageBox(Forijerapp::GetBCVersion(), _("O bc:"), wxOK, this);
}

void ForijerFrame::OnAboutGnuplot(wxCommandEvent &) {
	wxMessageBox(Forijerapp::GetGPVersion(), _("O gnuplot:"), wxOK, this);
}

void ForijerFrame::OnNacrtaj(wxCommandEvent &) {
	Nacrtaj();
}

void ForijerFrame::OnSpin(wxSpinEvent &) {
	if (chkRedrawAtChange && chkRedrawAtChange->IsChecked()) Nacrtaj();
}

void ForijerFrame::OnNew(wxCommandEvent &) {
	txtA0->Clear();
	txtAn->Clear();
	txtBn->Clear();
}

void ForijerFrame::OnOpen(wxCommandEvent &) {
	if (dlgFileOpen.ShowModal() == wxID_OK) {
		if (!wxFileExists(dlgFileOpen.GetPath())) {
			ErrorBox(_("Datoteka ne postoji!"), _("Greska!"));
		} else {
			if (LoadFormulas(dlgFileOpen.GetPath()))
				lastfile = dlgFileOpen.GetPath();
		}
	}
}

void ForijerFrame::OnSave(wxCommandEvent &e) {
	if (lastfile.empty()) {
		OnSaveAs(e);
		return;
	}
	SaveFormulas(lastfile);
}

void ForijerFrame::OnSaveAs(wxCommandEvent &) {
	if (dlgFileSave.ShowModal() == wxID_OK) {
		if (SaveFormulas(dlgFileSave.GetPath()))
			lastfile = dlgFileSave.GetPath();
	}
}

void ForijerFrame::OnGnuplotParametri(wxCommandEvent &) {
	wxString tmp;
	tmp = wxGetTextFromUser(_("Unesi parametre za gnuplot:"), _("Gnuplot parametri"), gnuplot_params, this);
	if (tmp.size()) gnuplot_params = tmp;
}

void ForijerFrame::OnShowGPConsole(wxCommandEvent &) {
	gpc->Show();
}
