//#include <wx/txtstrm.h>
#include "gnuplot.h"

gnuplot *OpenGnuplot() {
	gnuplot *gp = new gnuplot();
	if (!gp->init()) {
		delete gp;
		return NULL;
	}
	return gp;
}


gnuplot::gnuplot() : wxProcess(wxPROCESS_REDIRECT)
{
	inited = false;
}

gnuplot::~gnuplot() {
	if (inited) {
		//Kill(pid, wxSIGTERM);
		sendcmd(_("exit"));
		Detach();
	}
}

void gnuplot::OnTerminate(int pid, int status) {
	fprintf(stderr, "Ubijen gnuplot! (pid: %d)\n", pid);

	inited = false;
}

bool gnuplot::sendcmd(const wxString &cmd) {
	if (!inited) {
		fprintf(stderr, "gnuplot nije inicijaliziran!\n");
		return false;
	}
	
	wxOutputStream *os = GetOutputStream();
	if (!os) {
		fprintf(stderr, "Ne mogu doci do output stream-a!\n");
		return false;
	}
	
	//wxTextOutputStream tos(*os);
	//tos << cmd << endl;
	os->Write(cmd.mb_str(wxConvUTF8), cmd.size());
	os->PutC('\n');

	return true;
}

wxString gnuplot::readfromstream(wxInputStream *s) {
	wxChar c;
	wxString msg;
	//wxTextInputStream tis(*s);
	while (s->CanRead() && (c=s->GetC())) {//(c=tis.GetChar())) {
		msg += c;
	}
	return msg;
}

bool gnuplot::hasdata() const {
	return IsInputAvailable();
}

wxString gnuplot::getdata() {
	if (!inited) {
		return _("[[ gnuplot nije inicijaliziran! ]]");
	}
	
	wxInputStream *datas = GetInputStream();
	if (!datas) {
		return _("[[ Ne mogu doci do input stream-a! ]]");
	}

	if (!IsInputAvailable()) {
		return _("[[ Nema podataka! ]]");
	}

	return readfromstream(datas);
}

bool gnuplot::haserror() const {
	return IsErrorAvailable();
}

wxString gnuplot::geterrors() {
	if (!inited && !IsErrorAvailable()) {
		return _("[[ gnuplot nije inicijaliziran! ]]");
	}
	
	wxInputStream *errs = GetErrorStream();
	if (!errs) {
		return _("[[ Ne mogu doci do error stream-a! ]]");
	}

	if (!IsErrorAvailable()) {
		return _("[[ Nema gresaka! :) ]]");
	}

	return readfromstream(errs);
}

bool gnuplot::init() {
	if (inited) {
		fprintf(stderr, "Vec je inicijaliziran gnuplot!\n");
		return true;
	}

	if ((pid = wxExecute(_(GNUPLOTCMD), wxEXEC_ASYNC, this)) <= 0) {
		fprintf(stderr, "Ne mogu pokrenuti gnuplot!\n");
		return false;
	}
	
	inited = true;
	return true;
}
