#include "gpconsole.h"

BEGIN_EVENT_TABLE( GPConsole, wxFrame )
	EVT_BUTTON(id_BtnExec, GPConsole::OnBtnExec)
	EVT_TIMER(id_TimerDataCheck, GPConsole::OnTimerDataCheck)
	EVT_TEXT_ENTER(id_TextCmd, GPConsole::OnTextCmdEnter)
	EVT_CLOSE(GPConsole::OnClose)
END_EVENT_TABLE();


GPConsole::GPConsole(wxFrame *parent, gnuplot *_gp)
	: wxFrame(parent, -1, _("Gnuplot Console")),
	  tmrDataChk(this, id_TimerDataCheck),
	  gp(_gp)
{
	wxBoxSizer *szMain = new wxBoxSizer(wxVERTICAL);

	wxButton *btn;
	wxBoxSizer *szBot = new wxBoxSizer(wxHORIZONTAL);
	szBot->Add(cmd = new wxTextCtrl(this, id_TextCmd, _(""), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER), 1, wxEXPAND);
	szBot->Add(btn = new wxButton(this, id_BtnExec, _("E&xec")), 0, wxEXPAND);
	btn->SetDefault();

	szMain->Add(screen = new wxHtmlWindow(this), 1, wxEXPAND);
	szMain->Add(szBot, 0, wxEXPAND);
	SetSizer(szMain);

	screen->SetBorders(2);
	
	tmrDataChk.Start(500);
}

void GPConsole::OnTimerDataCheck(wxTimerEvent &) {
	if (gp->haserror()) {
		wxString err = gp->geterrors();
		err.Replace(_("\n"), _("<br>"));
		screen->AppendToPage(_("<font color=\"red\"><b>") + err + _("</b></font><br>"));
		ScrollToBottom();
	}

	if (gp->hasdata()) {
		wxString dat = gp->getdata();
		dat.Replace(_("\n"), _("<br>"));
		screen->AppendToPage(dat + _("<br>"));
		ScrollToBottom();
	}
}

void GPConsole::ScrollToBottom() {
	int w, h, wu, yu;
	screen->GetScrollPixelsPerUnit(&wu, &yu);
	if (yu) {
		screen->GetVirtualSize(&w, &h);
		h-=screen->GetSize().GetHeight();
		if (h > 0) screen->Scroll(-1, h/yu);
	}
}

void GPConsole::OnTextCmdEnter(wxCommandEvent &) {
	ExecCmd();
}

void GPConsole::OnBtnExec(wxCommandEvent &) {
	ExecCmd();
}

void GPConsole::ExecCmd() {
	ExecutedCmd(cmd->GetValue());
	gp->sendcmd(cmd->GetValue());
	cmd->Clear();
}

void GPConsole::ExecutedCmd(const wxString &cmd) {
	screen->AppendToPage(_("<b>gnuplot></b> ") + cmd + _("<br>"));
	ScrollToBottom();
}

void GPConsole::OnClose(wxCloseEvent &event) {
	if (event.CanVeto()) {
		Show(false);
		event.Veto();
	}
}
